/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
package org.openmuc.jsml.transport;

import java.io.DataInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * 
 * Implements SML transport layer version 1
 * 
 * Extracts the next SML File from a DataInputStream
 * 
 * @author mzillgit
 * 
 */
public class MessageExtractor {

    private final DataInputStream inputStream;

    private final ByteBuffer smlMessageBuffer = ByteBuffer.allocate(100000);

    private byte[] smlMessage = null;

    private int crc = 0xffff;

    private final byte[] buffer = new byte[8];

    private int bufferEnd = 0;

    private final long timeoutInMillies;

    private boolean startSequenceFound = false;

    public MessageExtractor(DataInputStream inputStream, long timeoutInMillies) throws IOException {
        this.inputStream = inputStream;
        this.timeoutInMillies = timeoutInMillies;
        smlMessageBuffer.rewind();
    }

    public byte[] getSmlMessage() throws IOException {
        waitForStartSequence();
        waitForStopSequence();
        return smlMessage;
    }

    private void waitForStartSequence() throws IOException {
        while (!startSequenceFound) {

            fillBufferWithTimeout();

            if (checkForStartSequence()) {
                // reset CRC
                crc = 0xffff;

                clearBuffer();

                startSequenceFound = true;
                return;
            }

            dropByte();
        }
    }

    private void waitForStopSequence() throws IOException {
        boolean stopSequenceFound = false;

        while (!stopSequenceFound) {
            fillBufferWithTimeout();

            if (checkForEscapeSequence()) {
                if (checkForEscapedEscapeSequence()) {
                    for (int i = 4; i < 8; i++) {
                        incrementalCRC(buffer[i]);
                    }

                    bufferEnd = 4;
                    clearBuffer();
                }
                else if (checkForStartSequence()) {
                    // if message starts at beginning
                    // reset CRC
                    crc = 0xffff;

                    clearBuffer();

                    startSequenceFound = true;
                    smlMessageBuffer.rewind();
                }
                else {
                    // end sequence
                    stopSequenceFound = true;
                    int numStuffBytes = buffer[5];

                    for (int i = 0; i < 6; i++) {
                        incrementalCRC(buffer[i]);
                    }

                    int receivedCRC = (0xff & (buffer[6])) * 0x100 + (0xff & buffer[7]);

                    if ((buffer[4] != 0x1a) || (numStuffBytes > 3)) {
                        throw new IOException("Termination sequence is wrong");
                    }

                    crc ^= 0xffff;
                    crc = ((crc & 0xff) << 8) | ((crc & 0xff00) >> 8);

                    if (receivedCRC != crc) {
                        throw new IOException("wrong crc");
                    }

                    int bufferLength = smlMessageBuffer.position();

                    smlMessage = new byte[bufferLength - numStuffBytes];
                    smlMessageBuffer.rewind();
                    smlMessageBuffer.get(smlMessage);
                }
            }
            else {
                incrementalCRC(dropByte());
            }
        }
    }

    private void clearBuffer() {
        while (bufferEnd > 0) {
            incrementalCRC(dropByte());
        }
    }

    private byte dropByte() {
        byte droppedByte = buffer[0];

        for (int i = 0; i < (bufferEnd - 1); i++) {
            buffer[i] = buffer[i + 1];
        }

        bufferEnd--;

        if (startSequenceFound) {
            smlMessageBuffer.put(droppedByte);
        }

        return droppedByte;
    }

    private void incrementalCRC(byte msgByte) {
        crc = (crc >> 8) ^ Crc16.TABLE[(crc ^ msgByte) & 0xff];
    }

    private void fillBufferWithTimeout() throws IOException {
        long lastRead = System.currentTimeMillis();

        while (bufferEnd < 8) {

            if ((System.currentTimeMillis() - lastRead) > timeoutInMillies) {
                throw new IOException("Timeout");
            }

            if (inputStream.available() > 0) {
                buffer[bufferEnd] = inputStream.readByte();
                bufferEnd++;
                lastRead = System.currentTimeMillis();
            }
            else {
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                }
            }
        }
    }

    private boolean checkForEscapeSequence() {
        for (int i = 0; i < 4; i++) {
            if (buffer[i] != 0x1b) {
                return false;
            }
        }

        return true;
    }

    private boolean checkForEscapedEscapeSequence() {
        for (int i = 4; i < 8; i++) {
            if (buffer[i] != 0x1b) {
                return false;
            }
        }
        return true;
    }

    private boolean checkForStartSequence() {
        if (checkForEscapeSequence() == false) {
            return false;
        }

        for (int i = 4; i < 8; i++) {
            if (buffer[i] != 0x01) {
                return false;
            }
        }

        return true;
    }

}
